@@ -25,7 +25,6 @@
 #include <ogg/ogg.h>
 
 #include <gst/gst.h>
-#include <gst/base/gstflowcombiner.h>
 
 #include "gstoggstream.h"
 
@@ -89,7 +88,6 @@ struct _GstOggPad
 
   gint64 packetno;
   gint64 current_granule;
-  gint64 prev_granule;
   gint64 keyframe_granule;
 
   GstClockTime start_time;      /* the timestamp of the first sample */
@@ -97,13 +95,11 @@ struct _GstOggPad
   gint64 first_granule;         /* the granulepos of first page == first sample in next page */
   GstClockTime first_time;      /* the timestamp of the second page or granuletime of first page */
 
-  GstClockTime position;        /* position when last push occurred; used to detect when we
+  GstClockTime position;        /* position when last push occured; used to detect when we
                                  * need to send a newsegment update event for sparse streams */
 
   GList *continued;
 
-  gboolean need_header_flag;
-
   gboolean discont;
   GstFlowReturn last_ret;       /* last return of _pad_push() */
   gboolean is_eos;
@@ -131,8 +127,6 @@ struct _GstOggDemux
 
   GstPad *sinkpad;
 
-  GstFlowCombiner *flowcombiner;
-
   gint64 length;
   gint64 read_offset;
   gint64 offset;
@@ -140,9 +134,6 @@ struct _GstOggDemux
   gboolean pullmode;
   gboolean running;
 
-  gboolean have_group_id;
-  guint group_id;
-
   gboolean need_chains;
   gboolean resync;
 
@@ -150,8 +141,6 @@ struct _GstOggDemux
      useful for skewing when seeking */
   guint64 max_packet_size, max_page_size;
 
-  gboolean check_index_overflow;
-
   /* state */
   GMutex chain_lock;           /* we need the lock to protect the chains */
   GArray *chains;               /* list of chains we know */
@@ -165,6 +154,7 @@ struct _GstOggDemux
   GstSegment segment;
   guint32  seqnum;
 
+  GstEvent *event;
   GstEvent *newsegment;         /* pending newsegment to be sent from _loop */
 
   /* annodex stuff */
@@ -181,7 +171,6 @@ struct _GstOggDemux
   enum { PUSH_PLAYING, PUSH_DURATION, PUSH_BISECT1, PUSH_LINEAR1, PUSH_BISECT2, PUSH_LINEAR2 } push_state;
 
   GstClockTime push_seek_time_original_target;
-  GstClockTime push_seek_time_original_stop;
   GstClockTime push_seek_time_target;
   gint64 push_last_seek_offset;
   GstClockTime push_last_seek_time;
@@ -203,17 +192,6 @@ struct _GstOggDemux
 
   /* ogg stuff */
   ogg_sync_state sync;
-  long chunk_size;
-
-  /* Seek events set up by the streaming thread */
-  GstEvent *seek_event;
-  GThread *seek_event_thread;
-  GMutex seek_event_mutex;
-  GCond seek_event_cond;
-  gboolean seek_event_thread_stop;
-  gboolean seek_thread_started;
-  GCond thread_started_cond;
-  guint32 seek_event_drop_till;
 };
 
 struct _GstOggDemuxClass
@@ -221,6 +199,8 @@ struct _GstOggDemuxClass
   GstElementClass parent_class;
 };
 
+gboolean gst_ogg_demux_plugin_init (GstPlugin * plugin);
+
 G_END_DECLS
 
 #endif /* __GST_OGG_DEMUX_H__ */
