@@ -20,10 +20,6 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef __GST_AUDIO_AUDIO_H__
-#include <gst/audio/audio.h>
-#endif
-
 #ifndef __GST_AUDIO_SINK_H__
 #define __GST_AUDIO_SINK_H__
 
@@ -66,7 +62,7 @@ struct _GstAudioSink {
  * @unprepare: Undo operations done in prepare.
  * @close: Close the device.
  * @write: Write data to the device.
- * @delay: Return how many frames are still in the device. This is used to
+ * @delay: Return how many samples are still in the device. This is used to
  *         drive the synchronisation.
  * @reset: Returns as quickly as possible from a write and flush any pending
  *         samples from the device.
@@ -88,7 +84,7 @@ struct _GstAudioSinkClass {
   gboolean (*close)     (GstAudioSink *sink);
   /* write samples to the device */
   gint     (*write)     (GstAudioSink *sink, gpointer data, guint length);
-  /* get number of frames queued in the device */
+  /* get number of samples queued in the device */
   guint    (*delay)     (GstAudioSink *sink);
   /* reset the audio device, unblock from a write */
   void     (*reset)     (GstAudioSink *sink);
