@@ -234,6 +234,7 @@ static const FormatInfo formats[] = {
   {"video/x-vp6-alpha", "On2 VP6 with alpha", FLAG_VIDEO, ""},
   {"video/x-vp7", "On2 VP7", FLAG_VIDEO, ""},
   {"video/x-vp8", "VP8", FLAG_VIDEO, ""},
+  {"video/x-vp9", "VP9", FLAG_VIDEO, ""},
   {"video/x-zlib", "Lossless zlib video", FLAG_VIDEO, ""},
   {"video/x-zmbv", "Zip Motion Block video", FLAG_VIDEO, ""},
 
@@ -838,7 +839,7 @@ gst_pb_utils_get_decoder_description (const GstCaps * caps)
 
   tmp = copy_and_clean_caps (caps);
 
-  g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
+  g_return_val_if_fail (has_single_media_type (tmp), NULL);
 
   /* special-case RTP caps */
   if (caps_are_rtp_caps (tmp, "video", &str)) {
@@ -889,7 +890,7 @@ gst_pb_utils_get_encoder_description (const GstCaps * caps)
   g_return_val_if_fail (caps != NULL, NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
   tmp = copy_and_clean_caps (caps);
-  g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
+  g_return_val_if_fail (has_single_media_type (tmp), NULL);
 
   /* special-case RTP caps */
   if (caps_are_rtp_caps (tmp, "video", &str)) {
@@ -948,8 +949,9 @@ gst_pb_utils_get_element_description (const gchar * factory_name)
 /**
  * gst_pb_utils_add_codec_description_to_tag_list:
  * @taglist: a #GstTagList
- * @codec_tag: a GStreamer codec tag such as #GST_TAG_AUDIO_CODEC,
- *             #GST_TAG_VIDEO_CODEC or #GST_TAG_CODEC
+ * @codec_tag: (allow-none): a GStreamer codec tag such as #GST_TAG_AUDIO_CODEC,
+ *             #GST_TAG_VIDEO_CODEC or #GST_TAG_CODEC. If none is specified,
+ *             the function will attempt to detect the appropriate category.
  * @caps: the (fixed) #GstCaps for which a codec tag should be added.
  *
  * Adds a codec tag describing the format specified by @caps to @taglist.
@@ -965,9 +967,8 @@ gst_pb_utils_add_codec_description_to_tag_list (GstTagList * taglist,
 
   g_return_val_if_fail (taglist != NULL, FALSE);
   g_return_val_if_fail (GST_IS_TAG_LIST (taglist), FALSE);
-  g_return_val_if_fail (codec_tag != NULL, FALSE);
-  g_return_val_if_fail (gst_tag_exists (codec_tag), FALSE);
-  g_return_val_if_fail (gst_tag_get_type (codec_tag) == G_TYPE_STRING, FALSE);
+  g_return_val_if_fail (codec_tag == NULL || (gst_tag_exists (codec_tag)
+          && gst_tag_get_type (codec_tag) == G_TYPE_STRING), FALSE);
   g_return_val_if_fail (caps != NULL, FALSE);
   g_return_val_if_fail (GST_IS_CAPS (caps), FALSE);
 
@@ -975,6 +976,20 @@ gst_pb_utils_add_codec_description_to_tag_list (GstTagList * taglist,
   if (info == NULL)
     return FALSE;
 
+  /* Attempt to find tag classification */
+  if (codec_tag == NULL) {
+    if (info->flags & FLAG_CONTAINER)
+      codec_tag = GST_TAG_CONTAINER_FORMAT;
+    else if (info->flags & FLAG_AUDIO)
+      codec_tag = GST_TAG_AUDIO_CODEC;
+    else if (info->flags & FLAG_VIDEO)
+      codec_tag = GST_TAG_VIDEO_CODEC;
+    else if (info->flags & FLAG_SUB)
+      codec_tag = GST_TAG_SUBTITLE_CODEC;
+    else
+      codec_tag = GST_TAG_CODEC;
+  }
+
   desc = format_info_get_desc (info, caps);
   gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE, codec_tag, desc, NULL);
   g_free (desc);
@@ -1006,7 +1021,7 @@ gst_pb_utils_get_codec_description (const GstCaps * caps)
   g_return_val_if_fail (caps != NULL, NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
   tmp = copy_and_clean_caps (caps);
-  g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
+  g_return_val_if_fail (has_single_media_type (tmp), NULL);
 
   info = find_format_info (tmp);
 
