@@ -121,6 +121,7 @@ struct _GstRTSPConnection
   GSocket *write_socket;
   GSocket *socket0, *socket1;
   gboolean manual_http;
+  gboolean may_cancel;
   GCancellable *cancellable;
 
   gchar tunnelid[TUNNELID_LEN];
@@ -217,6 +218,7 @@ gst_rtsp_connection_create (const GstRTSPUrl * url, GstRTSPConnection ** conn)
 
   newconn = g_new0 (GstRTSPConnection, 1);
 
+  newconn->may_cancel = TRUE;
   newconn->cancellable = g_cancellable_new ();
   newconn->client = g_socket_client_new ();
 
@@ -647,10 +649,11 @@ gst_rtsp_connection_connect (GstRTSPConnection * conn, GTimeVal * timeout)
     uri = g_strdup_printf ("http://%s:%d%s%s%s", url->host, url_port,
         url->abspath, url->query ? "?" : "", url->query ? url->query : "");
   } else {
-    uri = gst_rtsp_url_get_request_uri (conn->url);
+    uri = gst_rtsp_url_get_request_uri (url);
   }
+
   connection = g_socket_client_connect_to_uri (conn->client,
-      uri, 0, conn->cancellable, &error);
+      uri, url_port, conn->cancellable, &error);
   if (connection == NULL)
     goto connect_failed;
 
@@ -924,11 +927,11 @@ fill_raw_bytes (GstRTSPConnection * conn, guint8 * buffer, guint size,
     gsize count = size - out;
     if (block)
       r = g_input_stream_read (conn->input_stream, (gchar *) & buffer[out],
-          count, conn->cancellable, err);
+          count, conn->may_cancel ? conn->cancellable : NULL, err);
     else
       r = g_pollable_input_stream_read_nonblocking (G_POLLABLE_INPUT_STREAM
           (conn->input_stream), (gchar *) & buffer[out], count,
-          conn->cancellable, err);
+          conn->may_cancel ? conn->cancellable : NULL, err);
 
     if (G_UNLIKELY (r < 0)) {
       if (out == 0) {
@@ -1662,12 +1665,14 @@ build_next (GstRTSPBuilder * builder, GstRTSPMessage * message,
         if (c == '$') {
           /* data message, prepare for the header */
           builder->state = STATE_DATA_HEADER;
+          conn->may_cancel = FALSE;
         } else if (c == '\n' || c == '\r') {
           /* skip \n and \r */
           builder->offset = 0;
         } else {
           builder->line = 0;
           builder->state = STATE_READ_LINES;
+          conn->may_cancel = FALSE;
         }
         break;
       }
@@ -1769,6 +1774,8 @@ build_next (GstRTSPBuilder * builder, GstRTSPMessage * message,
         gchar *session_cookie;
         gchar *session_id;
 
+        conn->may_cancel = TRUE;
+
         if (message->type == GST_RTSP_MESSAGE_DATA) {
           /* data messages don't have headers */
           res = GST_RTSP_OK;
@@ -2778,6 +2785,7 @@ struct _GstRTSPWatch
 
   GSource *readsrc;
   GSource *writesrc;
+  gboolean write_added;
 
   gboolean keep_running;
 
@@ -2939,7 +2947,12 @@ gst_rtsp_source_dispatch (GSource * source, GSourceFunc callback G_GNUC_UNUSED,
     gpointer user_data G_GNUC_UNUSED)
 {
   GstRTSPWatch *watch = (GstRTSPWatch *) source;
+  GstRTSPConnection *conn = watch->conn;
 
+  if (conn->initial_buffer != NULL) {
+    gst_rtsp_source_dispatch_read (G_POLLABLE_INPUT_STREAM (conn->input_stream),
+        watch);
+  }
   return watch->keep_running;
 }
 
@@ -2961,8 +2974,13 @@ gst_rtsp_source_dispatch_write (GPollableOutputStream * stream,
 
       /* get a new message from the queue */
       rec = g_queue_pop_tail (watch->messages);
-      if (rec == NULL)
+      if (rec == NULL) {
+        if (watch->write_added) {
+          g_source_remove_child_source ((GSource *) watch, watch->writesrc);
+          watch->write_added = FALSE;
+        }
         break;
+      }
 
       watch->messages_bytes -= rec->size;
 
@@ -3036,6 +3054,11 @@ gst_rtsp_source_finalize (GSource * source)
   watch->messages_bytes = 0;
   g_free (watch->write_data);
 
+  if (watch->readsrc)
+    g_source_unref (watch->readsrc);
+  if (watch->writesrc)
+    g_source_unref (watch->writesrc);
+
   g_mutex_clear (&watch->mutex);
 
   if (watch->notify)
@@ -3109,10 +3132,17 @@ gst_rtsp_watch_new (GstRTSPConnection * conn,
 void
 gst_rtsp_watch_reset (GstRTSPWatch * watch)
 {
-  if (watch->readsrc)
+  if (watch->readsrc) {
     g_source_remove_child_source ((GSource *) watch, watch->readsrc);
-  if (watch->writesrc)
-    g_source_remove_child_source ((GSource *) watch, watch->writesrc);
+    g_source_unref (watch->readsrc);
+  }
+  if (watch->writesrc) {
+    if (watch->write_added) {
+      g_source_remove_child_source ((GSource *) watch, watch->writesrc);
+      watch->write_added = FALSE;
+    }
+    g_source_unref (watch->writesrc);
+  }
 
   if (watch->conn->input_stream) {
     watch->readsrc =
@@ -3121,7 +3151,6 @@ gst_rtsp_watch_reset (GstRTSPWatch * watch)
     g_source_set_callback (watch->readsrc,
         (GSourceFunc) gst_rtsp_source_dispatch_read, watch, NULL);
     g_source_add_child_source ((GSource *) watch, watch->readsrc);
-    g_source_unref (watch->readsrc);
   } else {
     watch->readsrc = NULL;
   }
@@ -3132,8 +3161,7 @@ gst_rtsp_watch_reset (GstRTSPWatch * watch)
         (watch->conn->output_stream), NULL);
     g_source_set_callback (watch->writesrc,
         (GSourceFunc) gst_rtsp_source_dispatch_write, watch, NULL);
-    g_source_add_child_source ((GSource *) watch, watch->writesrc);
-    g_source_unref (watch->writesrc);
+    /* we add the write source when we actually have something to write */
   } else {
     watch->writesrc = NULL;
   }
@@ -3305,6 +3333,10 @@ gst_rtsp_watch_write_data (GstRTSPWatch * watch, const guint8 * data,
   /* make sure the main context will now also check for writability on the
    * socket */
   context = ((GSource *) watch)->context;
+  if (!watch->write_added) {
+    g_source_add_child_source ((GSource *) watch, watch->writesrc);
+    watch->write_added = TRUE;
+  }
 
   if (id != NULL)
     *id = rec->id;
