@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -1589,7 +1589,7 @@ static gboolean
 gst_tag_demux_send_new_segment (GstTagDemux * tagdemux)
 {
   GstEvent *event;
-  gint64 start, stop, time;
+  gint64 start, stop, position;
   GstSegment *seg = &tagdemux->priv->segment;
   GstSegment newseg;
 
@@ -1609,10 +1609,10 @@ gst_tag_demux_send_new_segment (GstTagDemux * tagdemux)
 
   start = seg->start;
   stop = seg->stop;
-  time = seg->time;
+  position = seg->time;
 
   g_return_val_if_fail (start != -1, FALSE);
-  g_return_val_if_fail (time != -1, FALSE);
+  g_return_val_if_fail (position != -1, FALSE);
 
   if (tagdemux->priv->strip_end > 0) {
     if (gst_tag_demux_get_upstream_size (tagdemux)) {
@@ -1640,10 +1640,10 @@ gst_tag_demux_send_new_segment (GstTagDemux * tagdemux)
     else
       start = 0;
 
-    if (time > tagdemux->priv->strip_start)
-      time -= tagdemux->priv->strip_start;
+    if (position > tagdemux->priv->strip_start)
+      position -= tagdemux->priv->strip_start;
     else
-      time = 0;
+      position = 0;
 
     if (stop != -1) {
       if (stop > tagdemux->priv->strip_start)
@@ -1658,7 +1658,7 @@ gst_tag_demux_send_new_segment (GstTagDemux * tagdemux)
   gst_segment_copy_into (seg, &newseg);
   newseg.start = start;
   newseg.stop = stop;
-  newseg.time = time;
+  newseg.position = position;
   event = gst_event_new_segment (&newseg);
 
   return gst_pad_push_event (tagdemux->priv->srcpad, event);
