@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef _GST_VIDEO_DECODER_H_
@@ -148,8 +148,8 @@ GstFlowReturn _gst_video_decoder_error (GstVideoDecoder *dec, gint weight,
 G_STMT_START {                                                              \
   gchar *__txt = _gst_element_error_printf text;                            \
   gchar *__dbg = _gst_element_error_printf debug;                           \
-  GstVideoDecoder *__dec = GST_VIDEO_DECODER (el);                   \
-  ret = _gst_video_decoder_error (__dec, w, GST_ ## domain ## _ERROR,    \
+  GstVideoDecoder *dec = GST_VIDEO_DECODER (el);                   \
+  ret = _gst_video_decoder_error (dec, w, GST_ ## domain ## _ERROR,    \
       GST_ ## domain ## _ERROR_ ## code, __txt, __dbg, __FILE__,            \
       GST_FUNCTION, __LINE__);                                              \
 } G_STMT_END
