@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -57,18 +57,26 @@
   (gpointer)(((guint8*)GET_COMP_DATA (comp)) + \
       GET_COMP_STRIDE(comp) * (line))
 
-#define GET_STRIDE()                 GET_PLANE_STRIDE (0)
 #define GET_LINE(line)               GET_PLANE_LINE (0, line)
 
 #define GET_Y_LINE(line)             GET_COMP_LINE(GST_VIDEO_COMP_Y, line)
 #define GET_U_LINE(line)             GET_COMP_LINE(GST_VIDEO_COMP_U, line)
 #define GET_V_LINE(line)             GET_COMP_LINE(GST_VIDEO_COMP_V, line)
+
+#define GET_R_LINE(line)             GET_COMP_LINE(GST_VIDEO_COMP_R, line)
+#define GET_G_LINE(line)             GET_COMP_LINE(GST_VIDEO_COMP_G, line)
+#define GET_B_LINE(line)             GET_COMP_LINE(GST_VIDEO_COMP_B, line)
+
 #define GET_A_LINE(line)             GET_COMP_LINE(GST_VIDEO_COMP_A, line)
 
-#define GET_Y_STRIDE()               GET_COMP_STRIDE(GST_VIDEO_COMP_Y)
-#define GET_U_STRIDE()               GET_COMP_STRIDE(GST_VIDEO_COMP_U)
-#define GET_V_STRIDE()               GET_COMP_STRIDE(GST_VIDEO_COMP_V)
-#define GET_A_STRIDE()               GET_COMP_STRIDE(GST_VIDEO_COMP_A)
+#define GET_UV_420(line, flags)                 \
+  (flags & GST_VIDEO_PACK_FLAG_INTERLACED ?     \
+   ((line & ~3) >> 1) + (line & 1) :            \
+   line >> 1)
+#define GET_UV_410(line, flags)                 \
+  (flags & GST_VIDEO_PACK_FLAG_INTERLACED ?     \
+   ((line & ~7) >> 2) + (line & 1) :            \
+   line >> 2)
 
 #define PACK_420 GST_VIDEO_FORMAT_AYUV, unpack_planar_420, 1, pack_planar_420
 static void
@@ -76,8 +84,10 @@ unpack_planar_420 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     gpointer dest, const gpointer data[GST_VIDEO_MAX_PLANES],
     const gint stride[GST_VIDEO_MAX_PLANES], gint x, gint y, gint width)
 {
-  video_orc_unpack_I420 (dest,
-      GET_Y_LINE (y), GET_U_LINE (y >> 1), GET_V_LINE (y >> 1), width);
+  gint uv = GET_UV_420 (y, flags);
+
+  video_orc_unpack_I420 (dest, GET_Y_LINE (y), GET_U_LINE (uv),
+      GET_V_LINE (uv), width);
 }
 
 static void
@@ -86,8 +96,10 @@ pack_planar_420 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     const gint stride[GST_VIDEO_MAX_PLANES], GstVideoChromaSite chroma_site,
     gint y, gint width)
 {
-  video_orc_pack_I420 (GET_Y_LINE (y),
-      GET_U_LINE (y >> 1), GET_V_LINE (y >> 1), src, width / 2);
+  gint uv = GET_UV_420 (y, flags);
+
+  video_orc_pack_I420 (GET_Y_LINE (y), GET_U_LINE (uv), GET_V_LINE (uv),
+      src, width / 2);
 }
 
 #define PACK_YUY2 GST_VIDEO_FORMAT_AYUV, unpack_YUY2, 1, pack_YUY2
@@ -306,13 +318,13 @@ pack_v210 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     y4 = s[4 * (i + 4) + 1] >> 6;
     y5 = s[4 * (i + 5) + 1] >> 6;
 
-    u0 = (s[4 * (i + 0) + 2] + s[4 * (i + 1) + 2] + 1) >> 7;
-    u1 = (s[4 * (i + 2) + 2] + s[4 * (i + 3) + 2] + 1) >> 7;
-    u2 = (s[4 * (i + 4) + 2] + s[4 * (i + 5) + 2] + 1) >> 7;
+    u0 = s[4 * (i + 0) + 2] >> 6;
+    u1 = s[4 * (i + 2) + 2] >> 6;
+    u2 = s[4 * (i + 4) + 2] >> 6;
 
-    v0 = (s[4 * (i + 0) + 3] + s[4 * (i + 1) + 3] + 1) >> 7;
-    v1 = (s[4 * (i + 2) + 3] + s[4 * (i + 3) + 3] + 1) >> 7;
-    v2 = (s[4 * (i + 4) + 3] + s[4 * (i + 5) + 3] + 1) >> 7;
+    v0 = s[4 * (i + 0) + 3] >> 6;
+    v1 = s[4 * (i + 2) + 3] >> 6;
+    v2 = s[4 * (i + 4) + 3] >> 6;
 
     a0 = u0 | (y0 << 10) | (v0 << 20);
     a1 = y1 | (u1 << 10) | (y2 << 20);
@@ -390,10 +402,8 @@ pack_Y41B (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     destY[i + 2] = s[i * 4 + 9];
     destY[i + 3] = s[i * 4 + 13];
 
-    destU[i >> 2] =
-        (s[i * 4 + 2] + s[i * 4 + 6] + s[i * 4 + 10] + s[i * 4 + 14] + 2) >> 2;
-    destV[i >> 2] =
-        (s[i * 4 + 3] + s[i * 4 + 7] + s[i * 4 + 11] + s[i * 4 + 15] + 2) >> 2;
+    destU[i >> 2] = s[i * 4 + 2];
+    destV[i >> 2] = s[i * 4 + 3];
   }
 
   if (i == width - 3) {
@@ -401,14 +411,14 @@ pack_Y41B (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     destY[i + 1] = s[i * 4 + 5];
     destY[i + 2] = s[i * 4 + 9];
 
-    destU[i >> 2] = (s[i * 4 + 2] + s[i * 4 + 6] + s[i * 4 + 10] + 1) / 3;
-    destV[i >> 2] = (s[i * 4 + 3] + s[i * 4 + 7] + s[i * 4 + 11] + 1) / 3;
+    destU[i >> 2] = s[i * 4 + 2];
+    destV[i >> 2] = s[i * 4 + 3];
   } else if (i == width - 2) {
     destY[i] = s[i * 4 + 1];
     destY[i + 1] = s[i * 4 + 5];
 
-    destU[i >> 2] = (s[i * 4 + 2] + s[i * 4 + 6] + 1) >> 1;
-    destV[i >> 2] = (s[i * 4 + 3] + s[i * 4 + 7] + 1) >> 1;
+    destU[i >> 2] = s[i * 4 + 2];
+    destV[i >> 2] = s[i * 4 + 3];
   } else if (i == width - 1) {
     destY[i + 1] = s[i * 4 + 5];
 
@@ -457,6 +467,26 @@ pack_Y444 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
       width);
 }
 
+#define PACK_GBR GST_VIDEO_FORMAT_ARGB, unpack_GBR, 1, pack_GBR
+static void
+unpack_GBR (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
+    gpointer dest, const gpointer data[GST_VIDEO_MAX_PLANES],
+    const gint stride[GST_VIDEO_MAX_PLANES], gint x, gint y, gint width)
+{
+  video_orc_unpack_Y444 (dest, GET_R_LINE (y), GET_G_LINE (y),
+      GET_B_LINE (y), width);
+}
+
+static void
+pack_GBR (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
+    const gpointer src, gint sstride, gpointer data[GST_VIDEO_MAX_PLANES],
+    const gint stride[GST_VIDEO_MAX_PLANES], GstVideoChromaSite chroma_site,
+    gint y, gint width)
+{
+  video_orc_pack_Y444 (GET_R_LINE (y), GET_G_LINE (y), GET_B_LINE (y), src,
+      width);
+}
+
 #define PACK_GRAY8 GST_VIDEO_FORMAT_AYUV, unpack_GRAY8, 1, pack_GRAY8
 static void
 unpack_GRAY8 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
@@ -847,8 +877,10 @@ unpack_NV12 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     gpointer dest, const gpointer data[GST_VIDEO_MAX_PLANES],
     const gint stride[GST_VIDEO_MAX_PLANES], gint x, gint y, gint width)
 {
+  gint uv = GET_UV_420 (y, flags);
+
   video_orc_unpack_NV12 (dest,
-      GET_PLANE_LINE (0, y), GET_PLANE_LINE (1, y >> 1), width / 2);
+      GET_PLANE_LINE (0, y), GET_PLANE_LINE (1, uv), width / 2);
 }
 
 static void
@@ -857,8 +889,10 @@ pack_NV12 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     const gint stride[GST_VIDEO_MAX_PLANES], GstVideoChromaSite chroma_site,
     gint y, gint width)
 {
-  video_orc_pack_NV12 (GET_PLANE_LINE (0, y),
-      GET_PLANE_LINE (1, y >> 1), src, width / 2);
+  gint uv = GET_UV_420 (y, flags);
+
+  video_orc_pack_NV12 (GET_PLANE_LINE (0, y), GET_PLANE_LINE (1, uv),
+      src, width / 2);
 }
 
 #define PACK_NV21 GST_VIDEO_FORMAT_AYUV, unpack_NV21, 1, pack_NV21
@@ -867,8 +901,10 @@ unpack_NV21 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     gpointer dest, const gpointer data[GST_VIDEO_MAX_PLANES],
     const gint stride[GST_VIDEO_MAX_PLANES], gint x, gint y, gint width)
 {
+  gint uv = GET_UV_420 (y, flags);
+
   video_orc_unpack_NV21 (dest,
-      GET_PLANE_LINE (0, y), GET_PLANE_LINE (1, y >> 1), width / 2);
+      GET_PLANE_LINE (0, y), GET_PLANE_LINE (1, uv), width / 2);
 }
 
 static void
@@ -877,8 +913,30 @@ pack_NV21 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     const gint stride[GST_VIDEO_MAX_PLANES], GstVideoChromaSite chroma_site,
     gint y, gint width)
 {
+  gint uv = GET_UV_420 (y, flags);
+
   video_orc_pack_NV21 (GET_PLANE_LINE (0, y),
-      GET_PLANE_LINE (1, y >> 1), src, width / 2);
+      GET_PLANE_LINE (1, uv), src, width / 2);
+}
+
+#define PACK_NV16 GST_VIDEO_FORMAT_AYUV, unpack_NV16, 1, pack_NV16
+static void
+unpack_NV16 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
+    gpointer dest, const gpointer data[GST_VIDEO_MAX_PLANES],
+    const gint stride[GST_VIDEO_MAX_PLANES], gint x, gint y, gint width)
+{
+  video_orc_unpack_NV12 (dest,
+      GET_PLANE_LINE (0, y), GET_PLANE_LINE (1, y), width / 2);
+}
+
+static void
+pack_NV16 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
+    const gpointer src, gint sstride, gpointer data[GST_VIDEO_MAX_PLANES],
+    const gint stride[GST_VIDEO_MAX_PLANES], GstVideoChromaSite chroma_site,
+    gint y, gint width)
+{
+  video_orc_pack_NV12 (GET_PLANE_LINE (0, y),
+      GET_PLANE_LINE (1, y), src, width / 2);
 }
 
 #define PACK_UYVP GST_VIDEO_FORMAT_AYUV64, unpack_UYVP, 1, pack_UYVP
@@ -937,8 +995,8 @@ pack_UYVP (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
 
     y0 = s[4 * (i + 0) + 1];
     y1 = s[4 * (i + 1) + 1];
-    u0 = (s[4 * (i + 0) + 2] + s[4 * (i + 1) + 2] + 1) >> 1;
-    v0 = (s[4 * (i + 0) + 3] + s[4 * (i + 1) + 3] + 1) >> 1;
+    u0 = s[4 * (i + 0) + 2];
+    v0 = s[4 * (i + 0) + 3];
 
     d[(i / 2) * 5 + 0] = u0 >> 8;
     d[(i / 2) * 5 + 1] = (u0 & 0xc0) | y0 >> 10;
@@ -954,9 +1012,10 @@ unpack_A420 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     gpointer dest, const gpointer data[GST_VIDEO_MAX_PLANES],
     const gint stride[GST_VIDEO_MAX_PLANES], gint x, gint y, gint width)
 {
-  video_orc_unpack_A420 (dest,
-      GET_Y_LINE (y), GET_U_LINE (y >> 1), GET_V_LINE (y >> 1), GET_A_LINE (y),
-      width);
+  gint uv = GET_UV_420 (y, flags);
+
+  video_orc_unpack_A420 (dest, GET_Y_LINE (y), GET_U_LINE (uv),
+      GET_V_LINE (uv), GET_A_LINE (y), width);
 }
 
 static void
@@ -965,8 +1024,10 @@ pack_A420 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     const gint stride[GST_VIDEO_MAX_PLANES], GstVideoChromaSite chroma_site,
     gint y, gint width)
 {
-  video_orc_pack_A420 (GET_Y_LINE (y),
-      GET_U_LINE (y >> 1), GET_V_LINE (y >> 1), GET_A_LINE (y), src, width / 2);
+  gint uv = GET_UV_420 (y, flags);
+
+  video_orc_pack_A420 (GET_Y_LINE (y), GET_U_LINE (uv), GET_V_LINE (uv),
+      GET_A_LINE (y), src, width / 2);
 }
 
 #define PACK_RGB8P GST_VIDEO_FORMAT_ARGB, unpack_RGB8P, 1, pack_RGB8P
@@ -989,6 +1050,52 @@ unpack_RGB8P (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
   }
 }
 
+static const guint32 std_palette_RGB8P[] = {
+  0xff000000, 0xff000033, 0xff000066, 0xff000099, 0xff0000cc, 0xff0000ff,
+  0xff003300, 0xff003333, 0xff003366, 0xff003399, 0xff0033cc, 0xff0033ff,
+  0xff006600, 0xff006633, 0xff006666, 0xff006699, 0xff0066cc, 0xff0066ff,
+  0xff009900, 0xff009933, 0xff009966, 0xff009999, 0xff0099cc, 0xff0099ff,
+  0xff00cc00, 0xff00cc33, 0xff00cc66, 0xff00cc99, 0xff00cccc, 0xff00ccff,
+  0xff00ff00, 0xff00ff33, 0xff00ff66, 0xff00ff99, 0xff00ffcc, 0xff00ffff,
+  0xff330000, 0xff330033, 0xff330066, 0xff330099, 0xff3300cc, 0xff3300ff,
+  0xff333300, 0xff333333, 0xff333366, 0xff333399, 0xff3333cc, 0xff3333ff,
+  0xff336600, 0xff336633, 0xff336666, 0xff336699, 0xff3366cc, 0xff3366ff,
+  0xff339900, 0xff339933, 0xff339966, 0xff339999, 0xff3399cc, 0xff3399ff,
+  0xff33cc00, 0xff33cc33, 0xff33cc66, 0xff33cc99, 0xff33cccc, 0xff33ccff,
+  0xff33ff00, 0xff33ff33, 0xff33ff66, 0xff33ff99, 0xff33ffcc, 0xff33ffff,
+  0xff660000, 0xff660033, 0xff660066, 0xff660099, 0xff6600cc, 0xff6600ff,
+  0xff663300, 0xff663333, 0xff663366, 0xff663399, 0xff6633cc, 0xff6633ff,
+  0xff666600, 0xff666633, 0xff666666, 0xff666699, 0xff6666cc, 0xff6666ff,
+  0xff669900, 0xff669933, 0xff669966, 0xff669999, 0xff6699cc, 0xff6699ff,
+  0xff66cc00, 0xff66cc33, 0xff66cc66, 0xff66cc99, 0xff66cccc, 0xff66ccff,
+  0xff66ff00, 0xff66ff33, 0xff66ff66, 0xff66ff99, 0xff66ffcc, 0xff66ffff,
+  0xff990000, 0xff990033, 0xff990066, 0xff990099, 0xff9900cc, 0xff9900ff,
+  0xff993300, 0xff993333, 0xff993366, 0xff993399, 0xff9933cc, 0xff9933ff,
+  0xff996600, 0xff996633, 0xff996666, 0xff996699, 0xff9966cc, 0xff9966ff,
+  0xff999900, 0xff999933, 0xff999966, 0xff999999, 0xff9999cc, 0xff9999ff,
+  0xff99cc00, 0xff99cc33, 0xff99cc66, 0xff99cc99, 0xff99cccc, 0xff99ccff,
+  0xff99ff00, 0xff99ff33, 0xff99ff66, 0xff99ff99, 0xff99ffcc, 0xff99ffff,
+  0xffcc0000, 0xffcc0033, 0xffcc0066, 0xffcc0099, 0xffcc00cc, 0xffcc00ff,
+  0xffcc3300, 0xffcc3333, 0xffcc3366, 0xffcc3399, 0xffcc33cc, 0xffcc33ff,
+  0xffcc6600, 0xffcc6633, 0xffcc6666, 0xffcc6699, 0xffcc66cc, 0xffcc66ff,
+  0xffcc9900, 0xffcc9933, 0xffcc9966, 0xffcc9999, 0xffcc99cc, 0xffcc99ff,
+  0xffcccc00, 0xffcccc33, 0xffcccc66, 0xffcccc99, 0xffcccccc, 0xffccccff,
+  0xffccff00, 0xffccff33, 0xffccff66, 0xffccff99, 0xffccffcc, 0xffccffff,
+  0xffff0000, 0xffff0033, 0xffff0066, 0xffff0099, 0xffff00cc, 0xffff00ff,
+  0xffff3300, 0xffff3333, 0xffff3366, 0xffff3399, 0xffff33cc, 0xffff33ff,
+  0xffff6600, 0xffff6633, 0xffff6666, 0xffff6699, 0xffff66cc, 0xffff66ff,
+  0xffff9900, 0xffff9933, 0xffff9966, 0xffff9999, 0xffff99cc, 0xffff99ff,
+  0xffffcc00, 0xffffcc33, 0xffffcc66, 0xffffcc99, 0xffffcccc, 0xffffccff,
+  0xffffff00, 0xffffff33, 0xffffff66, 0xffffff99, 0xffffffcc, 0xffffffff,
+  0x00000000, 0xff000000, 0xff000000, 0xff000000, 0xff000000, 0xff000000,
+  0xff000000, 0xff000000, 0xff000000, 0xff000000, 0xff000000, 0xff000000,
+  0xff000000, 0xff000000, 0xff000000, 0xff000000, 0xff000000, 0xff000000,
+  0xff000000, 0xff000000, 0xff000000, 0xff000000, 0xff000000, 0xff000000,
+  0xff000000, 0xff000000, 0xff000000, 0xff000000, 0xff000000, 0xff000000,
+  0xff000000, 0xff000000, 0xff000000, 0xff000000, 0xff000000, 0xff000000,
+  0xff000000, 0xff000000, 0xff000000, 0xff000000
+};
+
 static void
 pack_RGB8P (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     const gpointer src, gint sstride, gpointer data[GST_VIDEO_MAX_PLANES],
@@ -1017,8 +1124,10 @@ unpack_410 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     gpointer dest, const gpointer data[GST_VIDEO_MAX_PLANES],
     const gint stride[GST_VIDEO_MAX_PLANES], gint x, gint y, gint width)
 {
+  gint uv = GET_UV_410 (y, flags);
+
   video_orc_unpack_YUV9 (dest,
-      GET_Y_LINE (y), GET_U_LINE (y >> 2), GET_V_LINE (y >> 2), width / 2);
+      GET_Y_LINE (y), GET_U_LINE (uv), GET_V_LINE (uv), width / 2);
 }
 
 static void
@@ -1028,9 +1137,10 @@ pack_410 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     gint y, gint width)
 {
   int i;
+  gint uv = GET_UV_410 (y, flags);
   guint8 *destY = GET_Y_LINE (y);
-  guint8 *destU = GET_U_LINE (y >> 2);
-  guint8 *destV = GET_V_LINE (y >> 2);
+  guint8 *destU = GET_U_LINE (uv);
+  guint8 *destV = GET_V_LINE (uv);
   const guint8 *s = src;
 
   for (i = 0; i < width - 3; i += 4) {
@@ -1039,10 +1149,8 @@ pack_410 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     destY[i + 2] = s[i * 4 + 9];
     destY[i + 3] = s[i * 4 + 13];
     if (y % 4 == 0) {
-      destU[i >> 2] =
-          (s[i * 4 + 2] + s[i * 4 + 6] + s[i * 4 + 10] + s[i * 4 + 14]) >> 2;
-      destV[i >> 2] =
-          (s[i * 4 + 3] + s[i * 4 + 7] + s[i * 4 + 11] + s[i * 4 + 15]) >> 2;
+      destU[i >> 2] = s[i * 4 + 2];
+      destV[i >> 2] = s[i * 4 + 3];
     }
   }
 
@@ -1051,15 +1159,15 @@ pack_410 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     destY[i + 1] = s[i * 4 + 5];
     destY[i + 2] = s[i * 4 + 9];
     if (y % 4 == 0) {
-      destU[i >> 2] = (s[i * 4 + 2] + s[i * 4 + 6] + s[i * 4 + 10]) / 3;
-      destV[i >> 2] = (s[i * 4 + 3] + s[i * 4 + 7] + s[i * 4 + 11]) / 3;
+      destU[i >> 2] = s[i * 4 + 2];
+      destV[i >> 2] = s[i * 4 + 3];
     }
   } else if (i == width - 2) {
     destY[i] = s[i * 4 + 1];
     destY[i + 1] = s[i * 4 + 5];
     if (y % 4 == 0) {
-      destU[i >> 2] = (s[i * 4 + 2] + s[i * 4 + 6]) >> 1;
-      destV[i >> 2] = (s[i * 4 + 3] + s[i * 4 + 7]) >> 1;
+      destU[i >> 2] = s[i * 4 + 2];
+      destV[i >> 2] = s[i * 4 + 3];
     }
   } else if (i == width - 1) {
     destY[i] = s[i * 4 + 1];
@@ -1132,23 +1240,21 @@ pack_IYU1 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     d[(i >> 2) * 6 + 2] = s[i * 4 + 5];
     d[(i >> 2) * 6 + 4] = s[i * 4 + 9];
     d[(i >> 2) * 6 + 5] = s[i * 4 + 13];
-    d[(i >> 2) * 6 + 0] =
-        (s[i * 4 + 2] + s[i * 4 + 6] + s[i * 4 + 10] + s[i * 4 + 14]) >> 2;
-    d[(i >> 2) * 6 + 3] =
-        (s[i * 4 + 3] + s[i * 4 + 7] + s[i * 4 + 11] + s[i * 4 + 15]) >> 2;
+    d[(i >> 2) * 6 + 0] = s[i * 4 + 2];
+    d[(i >> 2) * 6 + 3] = s[i * 4 + 3];
   }
 
   if (i == width - 3) {
     d[(i >> 2) * 6 + 1] = s[i * 4 + 1];
     d[(i >> 2) * 6 + 2] = s[i * 4 + 5];
     d[(i >> 2) * 6 + 4] = s[i * 4 + 9];
-    d[(i >> 2) * 6 + 0] = (s[i * 4 + 2] + s[i * 4 + 6] + s[i * 4 + 10]) / 3;
-    d[(i >> 2) * 6 + 3] = (s[i * 4 + 3] + s[i * 4 + 7] + s[i * 4 + 11]) / 3;
+    d[(i >> 2) * 6 + 0] = s[i * 4 + 2];
+    d[(i >> 2) * 6 + 3] = s[i * 4 + 3];
   } else if (i == width - 2) {
     d[(i >> 2) * 6 + 1] = s[i * 4 + 1];
     d[(i >> 2) * 6 + 2] = s[i * 4 + 5];
-    d[(i >> 2) * 6 + 0] = (s[i * 4 + 2] + s[i * 4 + 6]) >> 1;
-    d[(i >> 2) * 6 + 3] = (s[i * 4 + 3] + s[i * 4 + 7]) >> 1;
+    d[(i >> 2) * 6 + 0] = s[i * 4 + 2];
+    d[(i >> 2) * 6 + 3] = s[i * 4 + 3];
   } else if (i == width - 1) {
     d[(i >> 2) * 6 + 1] = s[i * 4 + 1];
     d[(i >> 2) * 6 + 0] = s[i * 4 + 2];
@@ -1224,6 +1330,210 @@ pack_r210 (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
   }
 }
 
+#define PACK_GBR_10LE GST_VIDEO_FORMAT_ARGB64, unpack_GBR_10LE, 1, pack_GBR_10LE
+static void
+unpack_GBR_10LE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
+    gpointer dest, const gpointer data[GST_VIDEO_MAX_PLANES],
+    const gint stride[GST_VIDEO_MAX_PLANES], gint x, gint y, gint width)
+{
+  int i;
+  guint16 *srcG = GET_G_LINE (y);
+  guint16 *srcB = GET_B_LINE (y);
+  guint16 *srcR = GET_R_LINE (y);
+  guint16 *d = dest, G, B, R;
+
+  for (i = 0; i < width; i++) {
+    G = GST_READ_UINT16_LE (srcG + i) << 6;
+    B = GST_READ_UINT16_LE (srcB + i) << 6;
+    R = GST_READ_UINT16_LE (srcR + i) << 6;
+
+    d[i * 4 + 0] = 0xffff;
+    d[i * 4 + 1] = R;
+    d[i * 4 + 2] = G;
+    d[i * 4 + 3] = B;
+  }
+}
+
+static void
+pack_GBR_10LE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
+    const gpointer src, gint sstride, gpointer data[GST_VIDEO_MAX_PLANES],
+    const gint stride[GST_VIDEO_MAX_PLANES], GstVideoChromaSite chroma_site,
+    gint y, gint width)
+{
+  int i;
+  guint16 *destG = GET_G_LINE (y);
+  guint16 *destB = GET_B_LINE (y);
+  guint16 *destR = GET_R_LINE (y);
+  guint16 G, B, R;
+  const guint16 *s = src;
+
+  for (i = 0; i < width; i++) {
+    G = (s[i * 4 + 2]) >> 6;
+    B = (s[i * 4 + 3]) >> 6;
+    R = (s[i * 4 + 1]) >> 6;
+
+    GST_WRITE_UINT16_LE (destG + i, G);
+    GST_WRITE_UINT16_LE (destB + i, B);
+    GST_WRITE_UINT16_LE (destR + i, R);
+  }
+}
+
+#define PACK_GBR_10BE GST_VIDEO_FORMAT_ARGB64, unpack_GBR_10BE, 1, pack_GBR_10BE
+static void
+unpack_GBR_10BE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
+    gpointer dest, const gpointer data[GST_VIDEO_MAX_PLANES],
+    const gint stride[GST_VIDEO_MAX_PLANES], gint x, gint y, gint width)
+{
+  int i;
+  guint16 *srcG = GET_G_LINE (y);
+  guint16 *srcB = GET_B_LINE (y);
+  guint16 *srcR = GET_R_LINE (y);
+  guint16 *d = dest, G, B, R;
+
+  for (i = 0; i < width; i++) {
+    G = GST_READ_UINT16_BE (srcG + i) << 6;
+    B = GST_READ_UINT16_BE (srcB + i) << 6;
+    R = GST_READ_UINT16_BE (srcR + i) << 6;
+
+    d[i * 4 + 0] = 0xffff;
+    d[i * 4 + 1] = R;
+    d[i * 4 + 2] = G;
+    d[i * 4 + 3] = B;
+  }
+}
+
+static void
+pack_GBR_10BE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
+    const gpointer src, gint sstride, gpointer data[GST_VIDEO_MAX_PLANES],
+    const gint stride[GST_VIDEO_MAX_PLANES], GstVideoChromaSite chroma_site,
+    gint y, gint width)
+{
+  int i;
+  guint16 *destG = GET_G_LINE (y);
+  guint16 *destB = GET_B_LINE (y);
+  guint16 *destR = GET_R_LINE (y);
+  guint16 G, B, R;
+  const guint16 *s = src;
+
+  for (i = 0; i < width; i++) {
+    G = s[i * 4 + 2] >> 6;
+    B = s[i * 4 + 3] >> 6;
+    R = s[i * 4 + 1] >> 6;
+
+    GST_WRITE_UINT16_BE (destG + i, G);
+    GST_WRITE_UINT16_BE (destB + i, B);
+    GST_WRITE_UINT16_BE (destR + i, R);
+  }
+}
+
+#define PACK_Y444_10LE GST_VIDEO_FORMAT_AYUV64, unpack_Y444_10LE, 1, pack_Y444_10LE
+static void
+unpack_Y444_10LE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
+    gpointer dest, const gpointer data[GST_VIDEO_MAX_PLANES],
+    const gint stride[GST_VIDEO_MAX_PLANES], gint x, gint y, gint width)
+{
+  int i;
+  guint16 *srcY = GET_Y_LINE (y);
+  guint16 *srcU = GET_U_LINE (y);
+  guint16 *srcV = GET_V_LINE (y);
+  guint16 *d = dest, Y, U, V;
+
+  for (i = 0; i < width; i++) {
+    Y = GST_READ_UINT16_LE (srcY + i) << 6;
+    U = GST_READ_UINT16_LE (srcU + i) << 6;
+    V = GST_READ_UINT16_LE (srcV + i) << 6;
+
+    if (!(flags & GST_VIDEO_PACK_FLAG_TRUNCATE_RANGE)) {
+      Y |= (Y >> 10);
+      U |= (U >> 10);
+      V |= (V >> 10);
+    }
+
+    d[i * 4 + 0] = 0xffff;
+    d[i * 4 + 1] = Y;
+    d[i * 4 + 2] = U;
+    d[i * 4 + 3] = V;
+  }
+}
+
+static void
+pack_Y444_10LE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
+    const gpointer src, gint sstride, gpointer data[GST_VIDEO_MAX_PLANES],
+    const gint stride[GST_VIDEO_MAX_PLANES], GstVideoChromaSite chroma_site,
+    gint y, gint width)
+{
+  int i;
+  guint16 *destY = GET_Y_LINE (y);
+  guint16 *destU = GET_U_LINE (y);
+  guint16 *destV = GET_V_LINE (y);
+  guint16 Y, U, V;
+  const guint16 *s = src;
+
+  for (i = 0; i < width; i++) {
+    Y = (s[i * 4 + 1]) >> 6;
+    U = (s[i * 4 + 2]) >> 6;
+    V = (s[i * 4 + 3]) >> 6;
+
+    GST_WRITE_UINT16_LE (destY + i, Y);
+    GST_WRITE_UINT16_LE (destU + i, U);
+    GST_WRITE_UINT16_LE (destV + i, V);
+  }
+}
+
+#define PACK_Y444_10BE GST_VIDEO_FORMAT_AYUV64, unpack_Y444_10BE, 1, pack_Y444_10BE
+static void
+unpack_Y444_10BE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
+    gpointer dest, const gpointer data[GST_VIDEO_MAX_PLANES],
+    const gint stride[GST_VIDEO_MAX_PLANES], gint x, gint y, gint width)
+{
+  int i;
+  guint16 *srcY = GET_Y_LINE (y);
+  guint16 *srcU = GET_U_LINE (y);
+  guint16 *srcV = GET_V_LINE (y);
+  guint16 *d = dest, Y, U, V;
+
+  for (i = 0; i < width; i++) {
+    Y = GST_READ_UINT16_BE (srcY + i) << 6;
+    U = GST_READ_UINT16_BE (srcU + i) << 6;
+    V = GST_READ_UINT16_BE (srcV + i) << 6;
+
+    if (!(flags & GST_VIDEO_PACK_FLAG_TRUNCATE_RANGE)) {
+      Y |= (Y >> 10);
+      U |= (U >> 10);
+      V |= (V >> 10);
+    }
+
+    d[i * 4 + 0] = 0xffff;
+    d[i * 4 + 1] = Y;
+    d[i * 4 + 2] = U;
+    d[i * 4 + 3] = V;
+  }
+}
+
+static void
+pack_Y444_10BE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
+    const gpointer src, gint sstride, gpointer data[GST_VIDEO_MAX_PLANES],
+    const gint stride[GST_VIDEO_MAX_PLANES], GstVideoChromaSite chroma_site,
+    gint y, gint width)
+{
+  int i;
+  guint16 *destY = GET_Y_LINE (y);
+  guint16 *destU = GET_U_LINE (y);
+  guint16 *destV = GET_V_LINE (y);
+  guint16 Y, U, V;
+  const guint16 *s = src;
+
+  for (i = 0; i < width; i++) {
+    Y = s[i * 4 + 1] >> 6;
+    U = s[i * 4 + 2] >> 6;
+    V = s[i * 4 + 3] >> 6;
+
+    GST_WRITE_UINT16_BE (destY + i, Y);
+    GST_WRITE_UINT16_BE (destU + i, U);
+    GST_WRITE_UINT16_BE (destV + i, V);
+  }
+}
+
 #define PACK_I420_10LE GST_VIDEO_FORMAT_AYUV64, unpack_I420_10LE, 1, pack_I420_10LE
 static void
 unpack_I420_10LE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
@@ -1231,9 +1541,10 @@ unpack_I420_10LE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     const gint stride[GST_VIDEO_MAX_PLANES], gint x, gint y, gint width)
 {
   int i;
+  gint uv = GET_UV_420 (y, flags);
   guint16 *srcY = GET_Y_LINE (y);
-  guint16 *srcU = GET_U_LINE (y >> 1);
-  guint16 *srcV = GET_V_LINE (y >> 1);
+  guint16 *srcU = GET_U_LINE (uv);
+  guint16 *srcV = GET_V_LINE (uv);
   guint16 *d = dest, Y, U, V;
 
   for (i = 0; i < width; i++) {
@@ -1261,17 +1572,18 @@ pack_I420_10LE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     gint y, gint width)
 {
   int i;
+  gint uv = GET_UV_420 (y, flags);
   guint16 *destY = GET_Y_LINE (y);
-  guint16 *destU = GET_U_LINE (y >> 1);
-  guint16 *destV = GET_V_LINE (y >> 1);
+  guint16 *destU = GET_U_LINE (uv);
+  guint16 *destV = GET_V_LINE (uv);
   guint16 Y0, Y1, U, V;
   const guint16 *s = src;
 
   for (i = 0; i < width - 1; i += 2) {
-    Y0 = (s[i * 4 + 1]) >> 6;
-    Y1 = (s[i * 4 + 5]) >> 6;
-    U = ((s[i * 4 + 2] + s[i * 4 + 6] + 1) >> 1) >> 6;
-    V = ((s[i * 4 + 3] + s[i * 4 + 7] + 1) >> 1) >> 6;
+    Y0 = s[i * 4 + 1] >> 6;
+    Y1 = s[i * 4 + 5] >> 6;
+    U = s[i * 4 + 2] >> 6;
+    V = s[i * 4 + 3] >> 6;
 
     GST_WRITE_UINT16_LE (destY + i + 0, Y0);
     GST_WRITE_UINT16_LE (destY + i + 1, Y1);
@@ -1296,9 +1608,10 @@ unpack_I420_10BE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     const gint stride[GST_VIDEO_MAX_PLANES], gint x, gint y, gint width)
 {
   int i;
+  gint uv = GET_UV_420 (y, flags);
   guint16 *srcY = GET_Y_LINE (y);
-  guint16 *srcU = GET_U_LINE (y >> 1);
-  guint16 *srcV = GET_V_LINE (y >> 1);
+  guint16 *srcU = GET_U_LINE (uv);
+  guint16 *srcV = GET_V_LINE (uv);
   guint16 *d = dest, Y, U, V;
 
   for (i = 0; i < width; i++) {
@@ -1326,17 +1639,18 @@ pack_I420_10BE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
     gint y, gint width)
 {
   int i;
+  gint uv = GET_UV_420 (y, flags);
   guint16 *destY = GET_Y_LINE (y);
-  guint16 *destU = GET_U_LINE (y >> 1);
-  guint16 *destV = GET_V_LINE (y >> 1);
+  guint16 *destU = GET_U_LINE (uv);
+  guint16 *destV = GET_V_LINE (uv);
   guint16 Y0, Y1, U, V;
   const guint16 *s = src;
 
   for (i = 0; i < width - 1; i += 2) {
     Y0 = s[i * 4 + 1] >> 6;
     Y1 = s[i * 4 + 5] >> 6;
-    U = ((s[i * 4 + 2] + s[i * 4 + 6] + 1) >> 1) >> 6;
-    V = ((s[i * 4 + 3] + s[i * 4 + 7] + 1) >> 1) >> 6;
+    U = s[i * 4 + 2] >> 6;
+    V = s[i * 4 + 3] >> 6;
 
     GST_WRITE_UINT16_BE (destY + i + 0, Y0);
     GST_WRITE_UINT16_BE (destY + i + 1, Y1);
@@ -1398,10 +1712,10 @@ pack_I422_10LE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
   const guint16 *s = src;
 
   for (i = 0; i < width - 1; i += 2) {
-    Y0 = (s[i * 4 + 1]) >> 6;
-    Y1 = (s[i * 4 + 5]) >> 6;
-    U = ((s[i * 4 + 2] + s[i * 4 + 6] + 1) >> 1) >> 6;
-    V = ((s[i * 4 + 3] + s[i * 4 + 7] + 1) >> 1) >> 6;
+    Y0 = s[i * 4 + 1] >> 6;
+    Y1 = s[i * 4 + 5] >> 6;
+    U = s[i * 4 + 2] >> 6;
+    V = s[i * 4 + 3] >> 6;
 
     GST_WRITE_UINT16_LE (destY + i + 0, Y0);
     GST_WRITE_UINT16_LE (destY + i + 1, Y1);
@@ -1465,8 +1779,8 @@ pack_I422_10BE (const GstVideoFormatInfo * info, GstVideoPackFlags flags,
   for (i = 0; i < width - 1; i += 2) {
     Y0 = s[i * 4 + 1] >> 6;
     Y1 = s[i * 4 + 5] >> 6;
-    U = ((s[i * 4 + 2] + s[i * 4 + 6] + 1) >> 1) >> 6;
-    V = ((s[i * 4 + 3] + s[i * 4 + 7] + 1) >> 1) >> 6;
+    U = s[i * 4 + 2] >> 6;
+    V = s[i * 4 + 3] >> 6;
 
     GST_WRITE_UINT16_BE (destY + i + 0, Y0);
     GST_WRITE_UINT16_BE (destY + i + 1, Y1);
@@ -1519,7 +1833,7 @@ typedef struct
 #define PSTR488           { 4, 8, 8, 0 }
 #define PSTR8888          { 8, 8, 8, 8 }
 
-/* planes */
+/* planes, in what plane do we find component N */
 #define PLANE_NA          0, { 0, 0, 0, 0 }
 #define PLANE0            1, { 0, 0, 0, 0 }
 #define PLANE01           2, { 0, 1, 0, 0 }
@@ -1527,6 +1841,7 @@ typedef struct
 #define PLANE012          3, { 0, 1, 2, 0 }
 #define PLANE0123         4, { 0, 1, 2, 3 }
 #define PLANE021          3, { 0, 2, 1, 0 }
+#define PLANE201          3, { 2, 0, 1, 0 }
 
 /* offsets */
 #define OFFS0             { 0, 0, 0, 0 }
@@ -1547,7 +1862,7 @@ typedef struct
 #define OFFS104           { 1, 0, 4, 0 }
 #define OFFS2460          { 2, 4, 6, 0 }
 
-/* subsampling */
+/* subsampling, w_sub, h_sub */
 #define SUB410            { 0, 2, 2, 0 }, { 0, 2, 2, 0 }
 #define SUB411            { 0, 2, 2, 0 }, { 0, 0, 0, 0 }
 #define SUB420            { 0, 1, 1, 0 }, { 0, 1, 1, 0 }
@@ -1707,6 +2022,21 @@ static VideoFormat formats[] = {
       PSTR222, PLANE012, OFFS0, SUB422, PACK_I422_10BE),
   MAKE_YUV_LE_FORMAT (I422_10LE, "raw video", 0x00000000, DPTH10_10_10,
       PSTR222, PLANE012, OFFS0, SUB422, PACK_I422_10LE),
+  MAKE_YUV_FORMAT (Y444_10BE, "raw video", 0x00000000, DPTH10_10_10,
+      PSTR222, PLANE012, OFFS0, SUB444, PACK_Y444_10BE),
+  MAKE_YUV_LE_FORMAT (Y444_10LE, "raw video", 0x00000000, DPTH10_10_10,
+      PSTR222, PLANE012, OFFS0, SUB444, PACK_Y444_10LE),
+  MAKE_RGB_FORMAT (GBR, "raw video", DPTH888, PSTR111, PLANE201, OFFS0, SUB444,
+      PACK_GBR),
+  MAKE_RGB_FORMAT (GBR_10BE, "raw video", DPTH10_10_10, PSTR222, PLANE201,
+      OFFS0, SUB444,
+      PACK_GBR_10BE),
+  MAKE_RGB_LE_FORMAT (GBR_10LE, "raw video", DPTH10_10_10, PSTR222, PLANE201,
+      OFFS0, SUB444,
+      PACK_GBR_10LE),
+
+  MAKE_YUV_FORMAT (NV16, "raw video", GST_MAKE_FOURCC ('N', 'V', '1', '6'),
+      DPTH888, PSTR111, PLANE011, OFFS001, SUB422, PACK_NV16),
 };
 
 static GstVideoFormat
@@ -1807,11 +2137,11 @@ gst_video_format_from_rgb16_masks (int red_mask, int green_mask, int blue_mask)
  * @depth: the amount of bits used for a pixel
  * @bpp: the amount of bits used to store a pixel. This value is bigger than
  *   @depth
- * @endianness: the endianness of the masks
+ * @endianness: the endianness of the masks, #G_LITTLE_ENDIAN or #G_BIG_ENDIAN
  * @red_mask: the red mask
  * @green_mask: the green mask
  * @blue_mask: the blue mask
- * @alpha_mask: the optional alpha mask
+ * @alpha_mask: the alpha mask, or 0 if no alpha mask
  *
  * Find the #GstVideoFormat for the given parameters.
  *
@@ -1820,7 +2150,7 @@ gst_video_format_from_rgb16_masks (int red_mask, int green_mask, int blue_mask)
  */
 GstVideoFormat
 gst_video_format_from_masks (gint depth, gint bpp, gint endianness,
-    gint red_mask, gint green_mask, gint blue_mask, gint alpha_mask)
+    guint red_mask, guint green_mask, guint blue_mask, guint alpha_mask)
 {
   GstVideoFormat format;
 
@@ -1829,6 +2159,7 @@ gst_video_format_from_masks (gint depth, gint bpp, gint endianness,
     red_mask = GUINT32_TO_BE (red_mask);
     green_mask = GUINT32_TO_BE (green_mask);
     blue_mask = GUINT32_TO_BE (blue_mask);
+    alpha_mask = GUINT32_TO_BE (alpha_mask);
     endianness = G_BIG_ENDIAN;
     if (bpp == 24) {
       red_mask >>= 8;
@@ -1908,6 +2239,8 @@ gst_video_format_from_fourcc (guint32 fourcc)
       return GST_VIDEO_FORMAT_NV12;
     case GST_MAKE_FOURCC ('N', 'V', '2', '1'):
       return GST_VIDEO_FORMAT_NV21;
+    case GST_MAKE_FOURCC ('N', 'V', '1', '6'):
+      return GST_VIDEO_FORMAT_NV16;
     case GST_MAKE_FOURCC ('v', '3', '0', '8'):
       return GST_VIDEO_FORMAT_v308;
     case GST_MAKE_FOURCC ('Y', '8', '0', '0'):
@@ -2005,53 +2338,30 @@ gst_video_format_get_info (GstVideoFormat format)
   return &formats[format].info;
 }
 
-typedef struct
-{
-  const gchar *name;
-  GstVideoChromaSite site;
-} ChromaSiteInfo;
-
-static const ChromaSiteInfo chromasite[] = {
-  {"jpeg", GST_VIDEO_CHROMA_SITE_JPEG},
-  {"mpeg2", GST_VIDEO_CHROMA_SITE_MPEG2},
-  {"dv", GST_VIDEO_CHROMA_SITE_DV}
-};
-
 /**
- * gst_video_chroma_from_string:
- * @s: a chromasite string
+ * gst_video_format_get_palette:
+ * @format: a #GstVideoFormat
+ * @size: (out): size of the palette in bytes
  *
- * Convert @s to a #GstVideoChromaSite
+ * Get the default palette of @format. This the palette used in the pack
+ * function for paletted formats.
  *
- * Returns: a #GstVideoChromaSite or %GST_VIDEO_CHROMA_SITE_UNKNOWN when @s does
- * not contain a valid chroma description.
+ * Returns: the default palette of @format or %NULL when @format does not have a
+ * palette.
+ *
+ * Since: 1.2
  */
-GstVideoChromaSite
-gst_video_chroma_from_string (const gchar * s)
+gconstpointer
+gst_video_format_get_palette (GstVideoFormat format, gsize * size)
 {
-  gint i;
-  for (i = 0; i < G_N_ELEMENTS (chromasite); i++) {
-    if (g_str_equal (chromasite[i].name, s))
-      return chromasite[i].site;
-  }
-  return GST_VIDEO_CHROMA_SITE_UNKNOWN;
-}
+  g_return_val_if_fail (format < G_N_ELEMENTS (formats), NULL);
+  g_return_val_if_fail (size != NULL, NULL);
 
-/**
- * gst_video_chroma_to_string:
- * @site: a #GstVideoChromaSite
- *
- * Converts @site to its string representation.
- *
- * Returns: a string describing @site.
- */
-const gchar *
-gst_video_chroma_to_string (GstVideoChromaSite site)
-{
-  gint i;
-  for (i = 0; i < G_N_ELEMENTS (chromasite); i++) {
-    if (chromasite[i].site == site)
-      return chromasite[i].name;
+  switch (format) {
+    case GST_VIDEO_FORMAT_RGB8P:
+      *size = sizeof (std_palette_RGB8P);
+      return std_palette_RGB8P;
+    default:
+      return NULL;
   }
-  return NULL;
 }
